@@ -55,7 +55,7 @@ typedef struct _zend_object_handlers zend_object_handlers;
 
 typedef struct _zend_object_value {
 	zend_object_handle handle;
-	zend_object_handlers *handlers;
+	const zend_object_handlers *handlers;
 } zend_object_value;
 
 #endif /* ZEND_TYPES_H */
