@@ -38,6 +38,7 @@ extern zend_module_entry json_module_entry;
 #endif
 
 ZEND_BEGIN_MODULE_GLOBALS(json)
+	int encoder_depth;
 	int error_code;
 ZEND_END_MODULE_GLOBALS(json)
 
@@ -48,17 +49,34 @@ ZEND_END_MODULE_GLOBALS(json)
 #endif
 
 PHP_JSON_API void php_json_encode(smart_str *buf, zval *val, int options TSRMLS_DC);
-PHP_JSON_API void php_json_decode(zval *return_value, char *str, int str_len, zend_bool assoc, long depth TSRMLS_DC);
+PHP_JSON_API void php_json_decode_ex(zval *return_value, char *str, int str_len, int options, long depth TSRMLS_DC);
+extern zend_class_entry *php_json_serializable_ce;
 
+
+/* json_encode() options */
 #define PHP_JSON_HEX_TAG	(1<<0)
 #define PHP_JSON_HEX_AMP	(1<<1)
 #define PHP_JSON_HEX_APOS	(1<<2)
 #define PHP_JSON_HEX_QUOT	(1<<3)
 #define PHP_JSON_FORCE_OBJECT	(1<<4)
 #define PHP_JSON_NUMERIC_CHECK	(1<<5)
+#define PHP_JSON_UNESCAPED_SLASHES	(1<<6)
+#define PHP_JSON_PRETTY_PRINT	(1<<7)
+#define PHP_JSON_UNESCAPED_UNICODE	(1<<8)
+
+/* Internal flags */
+#define PHP_JSON_OUTPUT_ARRAY	0
+#define PHP_JSON_OUTPUT_OBJECT	1
+
+/* json_decode() options */
+#define PHP_JSON_OBJECT_AS_ARRAY	(1<<0)
+#define PHP_JSON_BIGINT_AS_STRING	(1<<1)
+
+static inline void php_json_decode(zval *return_value, char *str, int str_len, zend_bool assoc, long depth TSRMLS_DC)
+{
+	php_json_decode_ex(return_value, str, str_len, assoc ? PHP_JSON_OBJECT_AS_ARRAY : 0, depth TSRMLS_CC);
+}
 
-#define PHP_JSON_OUTPUT_ARRAY 0
-#define PHP_JSON_OUTPUT_OBJECT 1
 
 #endif  /* PHP_JSON_H */
 
