@@ -26,7 +26,7 @@
 #include <dmalloc.h>
 #endif
 
-#define PHP_API_VERSION 20090626
+#define PHP_API_VERSION 20100412
 #define PHP_HAVE_STREAMS
 #define YYDEBUG 0
 
@@ -60,6 +60,7 @@
 #	else
 #		define PHPAPI
 #	endif
+
 #	define THREAD_LS
 #	define PHP_DIR_SEPARATOR '/'
 #	define PHP_EOL "\n"
@@ -141,8 +142,14 @@ char *strtok_r(char *s, const char *delim, char **last);
 END_EXTERN_C()
 #endif
 
-#if !defined(HAVE_SOCKLEN_T) && !defined(__OpenBSD__)
+#ifndef HAVE_SOCKLEN_T
+# if PHP_WIN32
+typedef int socklen_t;
+# else
+#   if !defined(__OpenBSD__)
 typedef unsigned int socklen_t;
+#   endif
+# endif
 #endif
 
 #define CREATE_MUTEX(a, b)
@@ -177,7 +184,6 @@ typedef unsigned int socklen_t;
 #endif
 
 #include "zend_hash.h"
-#include "php3_compat.h"
 #include "zend_alloc.h"
 #include "zend_stack.h"
 
@@ -192,8 +198,6 @@ typedef unsigned int socklen_t;
 # endif
 #endif
 
-#include "safe_mode.h"
-
 #ifndef HAVE_STRERROR
 char *strerror(int);
 #endif
@@ -253,6 +257,11 @@ END_EXTERN_C()
 # endif
 #endif
 
+#if defined(__GNUC__) && __GNUC__ >= 4
+# define php_ignore_value(x) (({ __typeof__ (x) __x = (x); (void) __x; }))
+#else
+# define php_ignore_value(x) ((void) (x))
+#endif
 
 /* global variables */
 #if !defined(PHP_WIN32)
@@ -274,6 +283,7 @@ void phperror(char *error);
 PHPAPI int php_write(void *buf, uint size TSRMLS_DC);
 PHPAPI int php_printf(const char *format, ...) PHP_ATTRIBUTE_FORMAT(printf, 1,
 		2);
+PHPAPI int php_get_module_initialized(void);
 PHPAPI void php_log_err(char *log_message TSRMLS_DC);
 int Debug(char *format, ...) PHP_ATTRIBUTE_FORMAT(printf, 1, 2);
 int cfgparse(void);
@@ -327,6 +337,7 @@ PHPAPI int php_register_internal_extensions(TSRMLS_D);
 PHPAPI int php_mergesort(void *base, size_t nmemb, register size_t size, int (*cmp)(const void *, const void * TSRMLS_DC) TSRMLS_DC);
 PHPAPI void php_register_pre_request_shutdown(void (*func)(void *), void *userdata);
 PHPAPI void php_com_initialize(TSRMLS_D);
+PHPAPI char *php_get_current_user(TSRMLS_D);
 END_EXTERN_C()
 
 /* PHP-named Zend macro wrappers */
@@ -382,20 +393,7 @@ END_EXTERN_C()
 
 /* Output support */
 #include "main/php_output.h"
-#define PHPWRITE(str, str_len)		php_body_write((str), (str_len) TSRMLS_CC)
-#define PUTS(str)					do {			\
-	const char *__str = (str);						\
-	php_body_write(__str, strlen(__str) TSRMLS_CC);	\
-} while (0)
 
-#define PUTC(c)						(php_body_write(&(c), 1 TSRMLS_CC), (c))
-#define PHPWRITE_H(str, str_len)	php_header_write((str), (str_len) TSRMLS_CC)
-#define PUTS_H(str)					do {				\
-	const char *__str = (str);							\
-	php_header_write(__str, strlen(__str) TSRMLS_CC);	\
-} while (0)
-
-#define PUTC_H(c)					(php_header_write(&(c), 1 TSRMLS_CC), (c))
 
 #include "php_streams.h"
 #include "php_memory_streams.h"
@@ -449,10 +447,6 @@ END_EXTERN_C()
 #endif
 #endif /* !XtOffsetOf */
 
-#if SUHOSIN_PATCH
-#include "suhosin_patch.h"
-#endif
-
 #endif
 
 /*
